# Provider MVC Example App

## Bulbs App

The app consist of 3 bulbs every time a bulb clicked its state will be toggled between (ON/OFF) and if all bulbs are ON the bg color will be light otherwise it’s dark. The app has also a fab once clicked it toggle every bulb state.

![image](asset/bulbs.gif)

### Getting Started

For web running use: flutter run -d chrome

### Code

#### **Controller** class definition:

File(_bulb.dart_)

```
class Bulb {
  bool isOn;

  Bulb({this.isOn = false});
}
```

#### **Controller** class definition:

File(_bulb_controller.dart_)

```
import 'package:provider_mvc/controller.dart';
import '../models/bulb.dart';

class BulbController extends Controller {
  final _bulb = Bulb();

  void toggle() {
    _bulb.isOn = !_bulb.isOn;
    refresh();
  }

  bool get isOn => _bulb.isOn;
}
```

File(_home_controller.dart_)

```
import 'package:flutter/material.dart';
import 'package:provider_mvc/controller.dart';

class HomeController extends Controller {
  Color _bg = Colors.black;

  set bg(Color c) {
    _bg = c;
    refresh();
  }

  Color get bg => _bg;
}
```

#### **View** class definition.

File(_bulb_view.dart_)

```
import 'package:flutter/material.dart';
import '../controllers/home_controller.dart';
import 'package:provider_mvc/controller.dart';
import '../controllers/bulb_controller.dart';
import 'package:provider_mvc/view.dart';

class BulbView extends View<BulbController> {
  static bool get isAllOn {
    bool v = true;
    Controller.of<BulbController>().forEach((ctrl) => v &= ctrl.isOn);
    return v;
  }

  @override
  Widget builder(
      BuildContext context, BulbController controller, Widget widget) {
    return GestureDetector(
      onTap: () {
        controller.toggle();
        Controller.find<HomeController>('HOME').single.bg =
            BulbView.isAllOn ? Colors.yellow[100] : Colors.black;
      },
      child: Container(
        height: 200,
        width: 250,
        decoration: BoxDecoration(
          borderRadius: BorderRadius.only(
              topLeft: Radius.circular(100),
              topRight: Radius.circular(100),
              bottomLeft: Radius.circular(30),
              bottomRight: Radius.circular(30)),
          boxShadow: controller.isOn
              ? <BoxShadow>[
                  BoxShadow(
                      spreadRadius: 5, color: Colors.orange, blurRadius: 100)
                ]
              : null,
          color: controller.isOn ? Colors.yellow : Colors.white,
        ),
      ),
    );
  }

  @override
  BulbController create(BuildContext context) =>
      Controller.put<BulbController>(BulbController());
}
```

File(_home_screen.dart_)

```
import 'package:flutter/material.dart';
import 'package:provider_mvc/controller.dart';
import '../views/bulb_view.dart';
import '../controllers/bulb_controller.dart';
import '../controllers/home_controller.dart';
import 'package:provider_mvc/view.dart';

class HomeScreen extends View<HomeController> {
  @override
  Widget builder(
      BuildContext context, HomeController controller, Widget widget) {
    return Scaffold(
      backgroundColor: controller.bg,
      body: Center(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            BulbView(),
            BulbView(),
            BulbView(),
          ],
        ),
      ),
      floatingActionButton: FloatingActionButton(
        onPressed: () {
          Controller.of<BulbController>().forEach((ctrl) => ctrl.toggle());
          controller.bg = BulbView.isAllOn ? Colors.yellow[100] : Colors.black;
        },
        child: Icon(Icons.refresh),
      ),
    );
  }

  @override
  HomeController create(BuildContext context) =>
      Controller.put<HomeController>(HomeController('HOME'));
}
```

#### The main class.

File(_main.dart_)

```
import 'package:flutter/material.dart';
import 'view/home_screen.dart';

void main() => runApp(MainApp());

class MainApp extends StatelessWidget {
  @override
  Widget build(BuildContext context) => MaterialApp(
        home: HomeScreen(),
        debugShowCheckedModeBanner: false,
      );
}
```
